@@ -32,9 +32,7 @@ enum class GridSetup {
   FullCheck     // additionally consistency of redundant data
 };
 
-template<typename T=float>
-struct Ccp4 {
-  Grid<T> grid;
+struct Ccp4Base {
   DataStats hstats;  // data statistics read from / written to ccp4 map
   // stores raw headers if the grid was read from ccp4 map
   std::vector<int32_t> ccp4_header;
@@ -50,6 +48,9 @@ struct Ccp4 {
       swap_four_bytes(&value);
     return value;
   }
+  std::array<int, 3> header_3i32(int w) const {
+    return {{ header_i32(w), header_i32(w+1), header_i32(w+2) }};
+  }
   float header_float(int w) const {
     int32_t int_value = header_i32(w);
     float f;
@@ -81,6 +82,44 @@ struct Ccp4 {
     std::memcpy(header_word(w), str.c_str(), str.size());
   }
 
+  std::array<int, 3> axis_positions() const {
+    if (ccp4_header.empty())
+      return {{0, 1, 2}}; // assuming it's X,Y,Z
+    std::array<int, 3> pos{{-1, -1, -1}};
+    for (int i = 0; i != 3; ++i) {
+      int mapi = header_i32(17 + i);
+      if (mapi <= 0 || mapi > 3 || pos[mapi - 1] != -1)
+        fail("Incorrect MAPC/MAPR/MAPS records");
+      pos[mapi - 1] = i;
+    }
+    return pos;
+  }
+
+  double header_rfloat(int w) const { // rounded to 5 digits
+    return std::round(1e5 * header_float(w)) / 1e5;
+  }
+
+  Box<Fractional> get_extent() const {
+    Box<Fractional> box;
+    // cf. setup()
+    auto pos = axis_positions();
+    std::array<int, 3> start = header_3i32(5);
+    std::array<int, 3> size = header_3i32(1);
+    std::array<int, 3> sampl = header_3i32(8);
+    for (int i = 0; i < 3; ++i) {
+      double scale = 1. / sampl[i];
+      int p = pos[i];
+      box.minimum.at(i) = scale * start[p] - 1e-9;
+      box.maximum.at(i) = scale * (start[p] + size[p] - 1) + 1e-9;
+    }
+    return box;
+  }
+};
+
+template<typename T=float>
+struct Ccp4 : public Ccp4Base {
+  Grid<T> grid;
+
   // this function assumes that the whole unit cell is covered with offset 0
   void prepare_ccp4_header_except_mode_and_stats() {
     GroupOps ops;
@@ -160,29 +199,11 @@ struct Ccp4 {
       // NXSTART et al. must be 0
       header_i32(5) == 0 && header_i32(6) == 0 && header_i32(7) == 0 &&
       // MX == NX
-      header_i32(8) == grid.nu && header_i32(9) == grid.nv &&
-                                  header_i32(10) == grid.nw &&
+      header_i32(8) == grid.nu && header_i32(9) == grid.nv && header_i32(10) == grid.nw &&
       // just in case, check ORIGIN
       header_i32(50) == 0 && header_i32(51) == 0 && header_i32(52) == 0;
   }
 
-  std::array<int, 3> axis_positions() const {
-    if (ccp4_header.empty())
-      return {{0, 1, 2}}; // assuming it's X,Y,Z
-    std::array<int, 3> pos{{-1, -1, -1}};
-    for (int i = 0; i != 3; ++i) {
-      int mapi = header_i32(17 + i);
-      if (mapi <= 0 || mapi > 3 || pos[mapi - 1] != -1)
-        fail("Incorrect MAPC/MAPR/MAPS records");
-      pos[mapi - 1] = i;
-    }
-    return pos;
-  }
-
-  double header_rfloat(int w) const { // rounded to 5 digits
-    return std::round(1e5 * header_float(w)) / 1e5;
-  }
-
   template<typename Stream>
   void read_ccp4_header(Stream& f, const std::string& path) {
     const size_t hsize = 256;
@@ -240,16 +261,17 @@ struct Ccp4 {
 
   inline void read_ccp4_from_memory(const char* data, size_t size,
                                     const std::string& name) {
-    return read_ccp4_stream(MemoryStream{data, data + size}, name);
+    read_ccp4_stream(MemoryStream(data, size), name);
   }
 
   template<typename Input>
   void read_ccp4(Input&& input) {
     if (input.is_stdin())
-      return read_ccp4_stream(FileStream{stdin}, "stdin");
-    if (input.is_compressed())
-      return read_ccp4_stream(input.get_uncompressing_stream(), input.path());
-    return read_ccp4_file(input.path());
+      read_ccp4_stream(FileStream{stdin}, "stdin");
+    else if (input.is_compressed())
+      read_ccp4_stream(input.get_uncompressing_stream(), input.path());
+    else
+      read_ccp4_file(input.path());
   }
 
   void write_ccp4_map(const std::string& path) const;
@@ -282,7 +304,7 @@ void write_data(const std::vector<TMem>& content, FILE* f) {
   if (typeid(TMem) == typeid(TFile)) {
     size_t len = content.size();
     if (std::fwrite(content.data(), sizeof(TFile), len, f) != len)
-      fail("Failed to write data to the map file.");
+      sys_fail("Failed to write data to the map file");
   } else {
     constexpr size_t chunk_size = 64 * 1024;
     std::vector<TFile> work(chunk_size);
@@ -291,7 +313,7 @@ void write_data(const std::vector<TMem>& content, FILE* f) {
       for (size_t j = 0; j < len; ++j)
         work[j] = static_cast<TFile>(content[i+j]);
       if (std::fwrite(work.data(), sizeof(TFile), len, f) != len)
-        fail("Failed to write data to the map file.");
+        sys_fail("Failed to write data to the map file");
     }
   }
 }
@@ -347,10 +369,10 @@ double Ccp4<T>::setup(GridSetup mode, T default_value) {
   if (grid.axis_order == AxisOrder::XYZ || ccp4_header.empty())
     return max_error;
   // cell sampling does not change
-  int sampl[3] = { header_i32(8), header_i32(9), header_i32(10) };
+  std::array<int, 3> sampl = header_3i32(8);
   // get old metadata
   auto pos = axis_positions();
-  int start[3] = { header_i32(5), header_i32(6), header_i32(7) };
+  std::array<int, 3> start = header_3i32(5);
   int end[3] = { start[0] + grid.nu, start[1] + grid.nv, start[2] + grid.nw };
   // set new metadata
   if (mode == GridSetup::ReorderOnly) {
@@ -422,7 +444,7 @@ void Ccp4<T>::set_extent(const Box<Fractional>& box) {
   int nv = (int)std::floor(box.maximum.y * grid.nv) - v0 + 1;
   int nw = (int)std::floor(box.maximum.z * grid.nw) - w0 + 1;
   // set the data
-  std::vector<T> data(nu * nv * nw);
+  std::vector<T> data((size_t)nu * nv * nw);
   int idx = 0;
   for (int w = 0; w < nw; w++) // sections
     for (int v = 0; v < nv; v++) // rows
