@@ -1,7 +1,7 @@
 // Copyright 2017 Global Phasing Ltd.
 //
-// struct Document that represents the CIF file (but could be also
-// read from, for example, JSON file (CIF-JSON or mmJSON).
+// struct Document that represents the CIF file (but can be also
+// read from JSON file, such as CIF-JSON or mmJSON).
 
 #ifndef GEMMI_CIFDOC_HPP_
 #define GEMMI_CIFDOC_HPP_
@@ -992,7 +992,7 @@ inline void check_for_duplicates(const Document& d) {
   for (const Block& block : d.blocks) {
     bool ok = names.insert(gemmi::to_lower(block.name)).second;
     if (!ok && !block.name.empty())
-      fail(d.source, ": duplicate block name: ", block.name);
+      fail(d.source + ": duplicate block name: ", block.name);
   }
   // check for dups inside each block
   std::unordered_set<std::string> frame_names;
