@@ -7,7 +7,7 @@
 
 #include <cassert>
 #include <cstdint>   // for int32_t
-#include <cstdio>    // for FILE, fread
+#include <cstdio>    // for FILE, fprintf
 #include <cstring>   // for memcpy
 #include <cmath>     // for isnan
 #include <algorithm> // for sort, any_of
@@ -74,7 +74,7 @@ struct Mtz {
     std::string crystal_name;
     std::string dataset_name;
     UnitCell cell;
-    double wavelength;
+    double wavelength;  // 0 means not set
   };
 
   struct Column {
@@ -332,6 +332,46 @@ struct Mtz {
     return nullptr;
   }
 
+  Column* column_with_type_and_one_of_labels(char type,
+                                             std::initializer_list<const char*> labels) {
+    for (Column& col : columns)
+      if (col.type == type) {
+        for (const char* label : labels)
+          if (col.label == label)
+            return &col;
+      }
+    return nullptr;
+  }
+
+  Column* rfree_column() {
+    return column_with_type_and_one_of_labels('I',
+        {"FREE", "RFREE", "FREER", "FreeR_flag", "R-free-flags"});
+  }
+  const Column* rfree_column() const {
+    return const_cast<Mtz*>(this)->rfree_column();
+  }
+
+  Column* imean_column() {
+    return column_with_type_and_one_of_labels('J', {"IMEAN", "I", "IOBS", "I-obs"});
+  }
+  const Column* imean_column() const {
+    return const_cast<Mtz*>(this)->imean_column();
+  }
+
+  Column* iplus_column() {
+    return column_with_type_and_one_of_labels('K', {"I(+)", "IOBS(+)", "I-obs(+)"});
+  }
+  const Column* iplus_column() const {
+    return const_cast<Mtz*>(this)->iplus_column();
+  }
+
+  Column* iminus_column() {
+    return column_with_type_and_one_of_labels('K', {"I(-)", "IOBS(-)", "I-obs(-)"});
+  }
+  const Column* iminus_column() const {
+    return const_cast<Mtz*>(this)->iminus_column();
+  }
+
   bool has_data() const {
     return data.size() == columns.size() * nreflections;
   }
@@ -687,7 +727,7 @@ struct Mtz {
     std::vector<int> indices(nreflections);
     for (int i = 0; i != nreflections; ++i)
       indices[i] = i;
-    std::sort(indices.begin(), indices.end(), [&](int i, int j) {
+    std::stable_sort(indices.begin(), indices.end(), [&](int i, int j) {
       int a = i * (int) columns.size();
       int b = j * (int) columns.size();
       return data[a] < data[b] || (data[a] == data[b] && (
@@ -697,6 +737,19 @@ struct Mtz {
     return indices;
   }
 
+  bool sort() {
+    std::vector<int> indices = sorted_row_indices();
+    sort_order = {{1, 2, 3, 0, 0}};
+    if (std::is_sorted(indices.begin(), indices.end()))
+      return false;
+    std::vector<float> new_data(data.size());
+    size_t w = columns.size();
+    for (size_t i = 0; i != indices.size(); ++i)
+      std::memcpy(&new_data[i * w], &data[indices[i] * w], w * sizeof(float));
+    data.swap(new_data);
+    return true;
+  }
+
   Miller get_hkl(size_t offset) const {
     return {{(int)data[offset], (int)data[offset+1], (int)data[offset+2]}};
   }
@@ -870,7 +923,7 @@ namespace gemmi {
     if (len < 80) \
       std::memset(buf + len, ' ', 80 - len); \
     if (write(buf, 80, 1) != 1) \
-      fail("Writing MTZ file failed"); \
+      sys_fail("Writing MTZ file failed"); \
   } while(0)
 
 template<typename Write>
