@@ -41,6 +41,7 @@ struct MmcifOutputGroups {
   bool entity_poly_seq:1;
   bool tls:1;
   bool software:1;
+  bool group_pdb:1;  // include _atom_site.group_PDB
 
   explicit MmcifOutputGroups(bool all)
     : atoms(all), block_name(all), entry(all), database_status(all),
@@ -51,7 +52,7 @@ struct MmcifOutputGroups {
       struct_asym(all), origx(all), struct_conf(all), struct_sheet(all),
       struct_biol(all), assembly(all), conn(all), cis(all),
       scale(all), atom_type(all), entity_poly_seq(all), tls(all),
-      software(all) {}
+      software(all), group_pdb(false) {}
 };
 
 void update_mmcif_block(const Structure& st, cif::Block& block, MmcifOutputGroups groups);
@@ -163,7 +164,7 @@ inline std::string qchain(const std::string& s) {
 }
 
 
-inline void add_cif_atoms(const Structure& st, cif::Block& block) {
+inline void add_cif_atoms(const Structure& st, cif::Block& block, bool use_group_pdb) {
   // atom list
   cif::Loop& atom_loop = block.init_mmcif_loop("_atom_site.", {
       "id",
@@ -184,6 +185,8 @@ inline void add_cif_atoms(const Structure& st, cif::Block& block) {
       "auth_seq_id",
       "auth_asym_id",
       "pdbx_PDB_model_num"});
+  if (use_group_pdb)
+    atom_loop.tags.emplace(atom_loop.tags.begin(), "_atom_site.group_PDB");
   bool has_calc_flag = false;
   bool has_tls_group_id = false;
   for (const Model& model : st.models)
@@ -213,6 +216,8 @@ inline void add_cif_atoms(const Structure& st, cif::Block& block) {
         if (const Entity* ent = gemmi::find_entity(res.subchain, st.entities))
           entity_id = cif::quote(ent->name);
         for (const Atom& atom : res.atoms) {
+          if (use_group_pdb)
+            vv.emplace_back(res.het_flag != 'H' ? "ATOM" : "HETATM");
           vv.emplace_back(std::to_string(++serial));
           vv.emplace_back(atom.element.uname());
           vv.emplace_back(cif::quote(atom.name));
@@ -387,8 +392,8 @@ void write_struct_conn(const Structure& st, cif::Block& block) {
   if (use_ccp4_link_id)
     conn_loop.tags.push_back("_struct_conn.ccp4_link_id");
   for (const Connection& con : st.connections) {
-    const_CRA cra1 = st.models[0].find_cra(con.partner1);
-    const_CRA cra2 = st.models[0].find_cra(con.partner2);
+    const_CRA cra1 = st.models[0].find_cra(con.partner1, true);
+    const_CRA cra2 = st.models[0].find_cra(con.partner2, true);
     if (!cra1.residue || !cra2.residue)
       continue;
     const Atom* at1 = cra1.atom;
@@ -562,28 +567,31 @@ void update_mmcif_block(const Structure& st, cif::Block& block, MmcifOutputGroup
                           impl::string_or_qmark(dbref.accession_code),
                           impl::string_or_qmark(dbref.isoform)});
         for (const std::string& subchain : ent.subchains) {
+          auto strand_id = subs.find(subchain);
+          if (strand_id == subs.end())
+            continue;
+          // DbRef::label_seq_begin/end (_struct_ref_seq.seq_align_beg/end) is
+          // not filled in when reading PDB file, so we check it here.
           Residue::OptionalNum label_begin = dbref.label_seq_begin;
           Residue::OptionalNum label_end = dbref.label_seq_end;
-          ConstResidueSpan span = st.models[0].get_subchain(subchain);
-          if (!label_begin) {
+          if (!label_begin || !label_end) {
+            ConstResidueSpan span = st.models[0].get_subchain(subchain);
             try {
               label_begin = span.auth_seq_id_to_label(dbref.seq_begin);
               label_end = span.auth_seq_id_to_label(dbref.seq_end);
-            } catch (const std::runtime_error&) {}
+            } catch (const std::out_of_range&) {}
           }
-          auto strand_id = subs.find(subchain);
-          if (strand_id != subs.end())
-            seq_loop.add_row({std::to_string(++counter2),
-                              std::to_string(counter),
-                              strand_id->second,  // pdbx_strand_id
-                              std::to_string(*label_begin),
-                              std::to_string(*label_end),
-                              std::to_string(*dbref.db_begin.num),
-                              std::to_string(*dbref.db_end.num),
-                              dbref.seq_begin.num.str(),
-                              impl::pdbx_icode(dbref.seq_begin),
-                              dbref.seq_end.num.str(),
-                              impl::pdbx_icode(dbref.seq_end)});
+          seq_loop.add_row({std::to_string(++counter2),
+                            std::to_string(counter),
+                            strand_id->second,  // pdbx_strand_id
+                            label_begin.str(),
+                            label_end.str(),
+                            dbref.db_begin.num.str(),
+                            dbref.db_end.num.str(),
+                            dbref.seq_begin.num.str(),
+                            impl::pdbx_icode(dbref.seq_begin),
+                            dbref.seq_end.num.str(),
+                            impl::pdbx_icode(dbref.seq_end)});
         }
       }
   }
@@ -1112,7 +1120,7 @@ void update_mmcif_block(const Structure& st, cif::Block& block, MmcifOutputGroup
   }
 
   if (groups.atoms)
-    impl::add_cif_atoms(st, block);
+    impl::add_cif_atoms(st, block, groups.group_pdb);
 
   if (groups.tls && st.meta.has_tls()) {
     cif::Loop& loop = block.init_mmcif_loop("_pdbx_refine_tls.", {
@@ -1189,7 +1197,7 @@ void add_minimal_mmcif_data(const Structure& st, cif::Block& block) {
   block.set_pair("_symmetry.space_group_name_H-M",
                  cif::quote(st.spacegroup_hm));
   impl::write_ncs_oper(st, block);
-  impl::add_cif_atoms(st, block);
+  impl::add_cif_atoms(st, block, /*use_group_pdb=*/false);
 }
 
 } // namespace gemmi
