@@ -57,11 +57,13 @@ namespace gemmi {
 
 #define WRITE(...) do { \
     gf_snprintf(buf, 82, __VA_ARGS__); \
+    buf[80] = '\n'; \
     os.write(buf, 81); \
   } while(0)
 
 #define WRITEU(...) do { \
     gf_snprintf(buf, 82, __VA_ARGS__); \
+    buf[80] = '\n'; \
     for (int i_ = 0; i_ != 80; i_++) \
       if (buf[i_] >= 'a' && buf[i_] <= 'z') buf[i_] -= 0x20; \
     os.write(buf, 81); \
@@ -71,7 +73,7 @@ namespace gemmi {
     int length__ = gf_snprintf(buf, 82, __VA_ARGS__); \
     if (length__ < 80) \
       std::memset(buf + length__, ' ', 80 - length__); \
-    buf[81] = '\n'; \
+    buf[80] = '\n'; \
     os.write(buf, 81); \
   } while(0)
 
@@ -148,19 +150,19 @@ inline void write_multiline(std::ostream& os, const char* record_name,
   char buf[88]; // a few bytes extra, just in case
   const char *start = text.c_str();
   const char *end = find_last_break(start, lastcol-10);
-  WRITEU("%-6s    %-70.*s\n", record_name, static_cast<int>(end-start), start);
+  WRITEU("%-6s    %-70.*s", record_name, static_cast<int>(end-start), start);
   for (int n = 2; n < 1000 && *end != '\0'; ++n) {
     start = end;
     end = find_last_break(start, lastcol-11);
     int len = int(end - start);
-    WRITEU("%-6s %3d %-69.*s\n", record_name, n, len, start);
+    WRITEU("%-6s %3d %-69.*s", record_name, n, len, start);
   }
 }
 
 inline void write_cryst1(const Structure& st, std::ostream& os) {
   char buf[88];
   const UnitCell& cell = st.cell;
-  WRITE("CRYST1%9.3f%9.3f%9.3f%7.2f%7.2f%7.2f %-11s%4s          \n",
+  WRITE("CRYST1%9.3f%9.3f%9.3f%7.2f%7.2f%7.2f %-11s%4s          ",
         cell.a, cell.b, cell.c, cell.alpha, cell.beta, cell.gamma,
         st.spacegroup_hm.empty() ? "P 1" : st.spacegroup_hm.c_str(),
         st.get_info("_cell.Z_PDB").c_str());
@@ -169,7 +171,7 @@ inline void write_cryst1(const Structure& st, std::ostream& os) {
 inline void write_ncs_op(const NcsOp& op, std::ostream& os) {
   char buf[88];
   for (int i = 0; i < 3; ++i) {
-    WRITE("MTRIX%d %3.3s%10.6f%10.6f%10.6f %14.5f    %-21c\n", i+1,
+    WRITE("MTRIX%d %3.3s%10.6f%10.6f%10.6f %14.5f    %-21c", i+1,
           op.id.c_str(), op.tr.mat[i][0], op.tr.mat[i][1], op.tr.mat[i][2],
           op.tr.vec.at(i), op.given ? '1' : ' ');
   }
@@ -189,8 +191,8 @@ inline void write_ncs(const Structure& st, std::ostream& os) {
 inline void write_remarks(const Structure& st, std::ostream& os) {
   char buf[88];
   if (st.resolution > 0) {
-    WRITE("%-80s\n", "REMARK   2");
-    WRITE("REMARK   2 RESOLUTION. %7.2f %-49s\n", st.resolution, "ANGSTROMS.");
+    WRITE("%-80s", "REMARK   2");
+    WRITE("REMARK   2 RESOLUTION. %7.2f %-49s", st.resolution, "ANGSTROMS.");
   }
   if (!st.assemblies.empty()) {
     const char* preface[] = {
@@ -202,19 +204,19 @@ inline void write_remarks(const Structure& st, std::ostream& os) {
       "REMARK 350 CRYSTALLOGRAPHIC OPERATIONS ARE GIVEN."
     };
     for (const char* line : preface)
-      WRITE("%-80s\n", line);
+      WRITE("%-80s", line);
     int counter = 0;
     for (const Assembly& assem : st.assemblies) {
-      WRITE("%-80s\n", "REMARK 350");
-      WRITE("REMARK 350 BIOMOLECULE: %-56d\n", ++counter);
+      WRITE("%-80s", "REMARK 350");
+      WRITE("REMARK 350 BIOMOLECULE: %-56d", ++counter);
       if (assem.author_determined)
-        WRITEU("REMARK 350 AUTHOR DETERMINED BIOLOGICAL UNIT: %-34s\n",
+        WRITEU("REMARK 350 AUTHOR DETERMINED BIOLOGICAL UNIT: %-34s",
                assem.oligomeric_details.c_str());
       if (assem.software_determined) {
-        WRITEU("REMARK 350 SOFTWARE DETERMINED QUATERNARY STRUCTURE: %-27s\n",
+        WRITEU("REMARK 350 SOFTWARE DETERMINED QUATERNARY STRUCTURE: %-27s",
                assem.oligomeric_details.c_str());
         if (!assem.software_name.empty())
-          WRITEU("REMARK 350 SOFTWARE USED: %-54s\n",
+          WRITEU("REMARK 350 SOFTWARE USED: %-54s",
                  assem.software_name.c_str());
         if (!std::isnan(assem.absa))
           WRITELN("REMARK 350 TOTAL BURIED SURFACE AREA: %.0f ANGSTROM**2",
@@ -258,7 +260,7 @@ inline void write_remarks(const Structure& st, std::ostream& os) {
           const Transform& tr = oper.transform;
           for (int i = 0; i < 3; ++i)
             WRITE("REMARK 350   "
-                  "BIOMT%d %3d%10.6f%10.6f%10.6f %14.5f            \n",
+                  "BIOMT%d %3d%10.6f%10.6f%10.6f %14.5f            ",
                   i+1, oper_cnt,
                   tr.mat[i][0], tr.mat[i][1], tr.mat[i][2], tr.vec.at(i));
         }
@@ -272,6 +274,7 @@ inline void write_chain_atoms(const Chain& chain, std::ostream& os,
   char buf[88];
   char buf8[8];
   char buf8a[8];
+  buf[0] = '\0';
   if (chain.name.length() > 2)
     fail("long chain name: " + chain.name);
   for (const Residue& res : chain.residues) {
@@ -297,9 +300,9 @@ inline void write_chain_atoms(const Chain& chain, std::ostream& os,
       // 73-76      segment identifier, left-justified (non-standard)
       // 77-78  2s  element symbol, right-justified
       // 79-80  2s  charge
-      WRITE("%-6s%5s %-4s%c%3s"
+      WRITE("%-6s%5s %-4.4s%c%3s"
             "%2s%5s   %8.3f%8.3f%8.3f"
-            "%6.2f%6.2f      %-4.4s%2s%c%c\n",
+            "%6.2f%6.2f      %-4.4s%2s%c%c",
             as_het ? "HETATM" : "ATOM",
             impl::encode_serial_in_hybrid36(buf8, ++serial),
             padded_atom_name(a).c_str(),
@@ -307,7 +310,7 @@ inline void write_chain_atoms(const Chain& chain, std::ostream& os,
             res.name.c_str(),
             chain.name.c_str(),
             impl::write_seq_id(buf8a, res.seqid),
-            // We want to avoid negative zero and round them numbers up
+            // We want to avoid negative zero and round the numbers up
             // if they originally had one digit more and that digit was 5.
             a.pos.x > -5e-4 && a.pos.x < 0 ? 0 : a.pos.x + 1e-10,
             a.pos.y > -5e-4 && a.pos.y < 0 ? 0 : a.pos.y + 1e-10,
@@ -318,7 +321,7 @@ inline void write_chain_atoms(const Chain& chain, std::ostream& os,
             // B is harder to get rounded right. It is stored as float,
             // and may be given with more than single precision in mmCIF
             // If it was originally %.5f (5TIS) we need to add 0.5 * 10^-5.
-            a.b_iso + 0.5e-5,
+            std::min(a.b_iso + 0.5e-5, 999.99),
             res.segment.c_str(),
             a.element.uname(),
             // Charge is written as 1+ or 2-, etc, or just empty space.
@@ -337,7 +340,7 @@ inline void write_chain_atoms(const Chain& chain, std::ostream& os,
         os.write(buf, 81);
       }
     }
-    if (opt.ter_records &&
+    if (opt.ter_records && buf[0] != '\0' &&
         (opt.ter_ignores_type ? &res == &chain.residues.back()
                               : (res.entity_type == EntityType::Polymer &&
                                 (&res == &chain.residues.back() ||
@@ -351,7 +354,7 @@ inline void write_chain_atoms(const Chain& chain, std::ostream& os,
         std::memset(buf+28, ' ', 52);
         os.write(buf, 81);
       } else {
-        WRITE("%-80s\n", "TER");
+        WRITE("%-80s", "TER");
       }
     }
   }
@@ -370,12 +373,12 @@ inline void write_atoms(const Structure& st, std::ostream& os,
           name = std::to_string(&model - &st.models[0] + 1);
           break;
         }
-      WRITE("MODEL %8s %65s\n", name.c_str(), "");
+      WRITE("MODEL %8s %65s", name.c_str(), "");
     }
     for (const Chain& chain : model.chains)
       write_chain_atoms(chain, os, serial, opt);
     if (st.models.size() > 1)
-      WRITE("%-80s\n", "ENDMDL");
+      WRITE("%-80s", "ENDMDL");
   }
 }
 
@@ -396,7 +399,7 @@ inline void write_header(const Structure& st, std::ostream& os,
     // "classification" in PDB == _struct_keywords.pdbx_keywords in mmCIF
     const std::string& keywords = st.get_info("_struct_keywords.pdbx_keywords");
     if (!pdb_date.empty() || !keywords.empty() || !entry_id.empty())
-      WRITEU("HEADER    %-40.40s%-9s   %-18.18s\n",
+      WRITEU("HEADER    %-40.40s%-9s   %-18.18s",
              keywords.c_str(), pdb_date.c_str(), entry_id.c_str());
   }
   write_multiline(os, "TITLE", st.get_info("_struct.title"), 80);
@@ -407,7 +410,7 @@ inline void write_header(const Structure& st, std::ostream& os,
                       [](const ExperimentInfo& e) { return e.method; });
   write_multiline(os, "EXPDTA", expdta, 79);
   if (st.models.size() > 1)
-    WRITE("NUMMDL    %-6zu %63s\n", st.models.size(), "");
+    WRITE("NUMMDL    %-6zu %63s", st.models.size(), "");
 
   if (!st.raw_remarks.empty()) {
     for (const std::string& line : st.raw_remarks) {
@@ -475,7 +478,7 @@ inline void write_header(const Structure& st, std::ostream& os,
           }
           os.write(buf, 81);
           if (!short_record)
-            WRITE("DBREF2 %4s%2s     %-22s     %10d  %10d             \n",
+            WRITE("DBREF2 %4s%2s     %-22s     %10d  %10d             ",
                   entry_id.c_str(), ch.name.c_str(),
                   dbref.accession_code.c_str(), *begin.num, *end.num);
         }
@@ -536,7 +539,7 @@ inline void write_header(const Structure& st, std::ostream& os,
         const AtomAddress& a1 = strand.hbond_atom1;
         // H-bond atom names are expected to be O and N
         WRITE("SHEET%5d %3.3s%2zu %3s%2s%5s %3s%2s%5s%2d  %-3s%3s%2s%5s "
-              " %-3s%3s%2s%5s          \n",
+              " %-3s%3s%2s%5s          ",
               ++strand_counter, sheet.name.c_str(), sheet.strands.size(),
               strand.start.res_id.name.c_str(), strand.start.chain_name.c_str(),
               write_seq_id(buf8a, strand.start.res_id.seqid),
@@ -560,15 +563,15 @@ inline void write_header(const Structure& st, std::ostream& os,
       int counter = 0;
       for (const Connection& con : st.connections)
         if (con.type == Connection::Disulf) {
-          const_CRA cra1 = st.models[0].find_cra(con.partner1);
-          const_CRA cra2 = st.models[0].find_cra(con.partner2);
+          const_CRA cra1 = st.models[0].find_cra(con.partner1, true);
+          const_CRA cra2 = st.models[0].find_cra(con.partner2, true);
           if (!cra1.atom || !cra2.atom)
             continue;
           SymImage im = st.cell.find_nearest_image(cra1.atom->pos,
                                                    cra2.atom->pos, con.asu);
           if (++counter == 10000)
             counter = 0;
-          WRITE("SSBOND%4d %3s%2s %5s %5s%2s %5s %28s %6s %5.2f  \n",
+          WRITE("SSBOND%4d %3s%2s %5s %5s%2s %5s %28s %6s %5.2f  ",
              counter,
              cra1.residue->name.c_str(), cra1.chain->name.c_str(),
              write_seq_id(buf8, cra1.residue->seqid),
@@ -583,18 +586,16 @@ inline void write_header(const Structure& st, std::ostream& os,
       for (const Connection& con : st.connections)
         if (con.type == Connection::Covale || con.type == Connection::MetalC ||
             con.type == Connection::Unknown) {
-          const Residue* res1 = st.models[0].find_residue(con.partner1.chain_name, con.partner1.res_id);
-          const Residue* res2 = st.models[0].find_residue(con.partner2.chain_name, con.partner2.res_id);
-          if (!res1 || !res2)
+          const_CRA cra1 = st.models[0].find_cra(con.partner1, true);
+          const_CRA cra2 = st.models[0].find_cra(con.partner2, true);
+          // In special cases (LINKR gap) atoms are not there.
+          if (!cra1.residue || !cra2.residue)
             continue;
-          const Atom* at1 = res1->find_atom(con.partner1.atom_name, con.partner1.altloc);
-          const Atom* at2 = res2->find_atom(con.partner2.atom_name, con.partner2.altloc);
-          
           std::string im_pdb_symbol = "", im_dist_str = "";
           bool im_same_asu = true;
-          if (at1 && at2) {
-            SymImage im = st.cell.find_nearest_image(at1->pos,
-                                                     at2->pos, con.asu);
+          if (cra1.atom && cra2.atom) {
+            SymImage im = st.cell.find_nearest_image(cra1.atom->pos,
+                                                     cra2.atom->pos, con.asu);
             im_pdb_symbol = im.pdb_symbol(false);
             im_dist_str = to_str_prec<2>(im.dist());
             im_same_asu = im.same_asu();
@@ -606,16 +607,16 @@ inline void write_header(const Structure& st, std::ostream& os,
           // except for LINKR (Refmac variant of LINK).
           gf_snprintf(buf, 82, "LINK        %-4s%c%3s%2s%5s   "
                 "            %-4s%c%3s%2s%5s  %6s %6s %5s  \n",
-                at1 ? padded_atom_name(*at1).c_str() : "",
-                at1 && at1->altloc ? std::toupper(at1->altloc) : ' ',
-                res1->name.c_str(),
+                cra1.atom ? padded_atom_name(*cra1.atom).c_str() : "",
+                cra1.atom && cra1.atom->altloc ? std::toupper(cra1.atom->altloc) : ' ',
+                cra1.residue->name.c_str(),
                 con.partner1.chain_name.c_str(),
-                write_seq_id(buf8, res1->seqid),
-                at2 ? padded_atom_name(*at2).c_str() : "",
-                at2 && at2->altloc ? std::toupper(at2->altloc) : ' ',
-                res2->name.c_str(),
+                write_seq_id(buf8, cra1.residue->seqid),
+                cra2.atom ? padded_atom_name(*cra2.atom).c_str() : "",
+                cra2.atom && cra2.atom->altloc ? std::toupper(cra2.atom->altloc) : ' ',
+                cra2.residue->name.c_str(),
                 con.partner2.chain_name.c_str(),
-                write_seq_id(buf8a, res2->seqid),
+                write_seq_id(buf8a, cra2.residue->seqid),
                 "1555", im_pdb_symbol.c_str(), im_dist_str.c_str());
           if (opt.use_linkr && !con.link_id.empty()) {
             buf[4] = 'R';  // LINK -> LINKR
@@ -640,7 +641,7 @@ inline void write_header(const Structure& st, std::ostream& os,
               if (next && are_connected(res, *next, PolymerType::PeptideL)) {
                 if (++counter == 10000)
                   counter = 0;
-                WRITE("CISPEP%4d %3s%2s %5s   %3s%2s %5s %9s %12.2f %20s\n",
+                WRITE("CISPEP%4d %3s%2s %5s   %3s%2s %5s %9s %12.2f %20s",
                       counter,
                       res.name.c_str(), chain.name.c_str(),
                       write_seq_id(buf8, res.seqid),
@@ -657,14 +658,14 @@ inline void write_header(const Structure& st, std::ostream& os,
   write_cryst1(st, os);
   if (st.has_origx && !st.origx.is_identity()) {
     for (int i = 0; i < 3; ++i)
-      WRITE("ORIGX%d %13.6f%10.6f%10.6f %14.5f %24s\n", i+1,
+      WRITE("ORIGX%d %13.6f%10.6f%10.6f %14.5f %24s", i+1,
             st.origx.mat[i][0], st.origx.mat[i][1], st.origx.mat[i][2],
             st.origx.vec.at(i), "");
   }
   if (st.cell.explicit_matrices) {
     for (int i = 0; i < 3; ++i)
       // We add a small number to avoid negative 0.
-      WRITE("SCALE%d %13.6f%10.6f%10.6f %14.5f %24s\n", i+1,
+      WRITE("SCALE%d %13.6f%10.6f%10.6f %14.5f %24s", i+1,
             st.cell.frac.mat[i][0] + 1e-15, st.cell.frac.mat[i][1] + 1e-15,
             st.cell.frac.mat[i][2] + 1e-15, st.cell.frac.vec.at(i) + 1e-15, "");
   }
@@ -692,7 +693,7 @@ void write_pdb(const Structure& st, std::ostream& os, PdbWriteOptions opt) {
   impl::write_header(st, os, opt);
   impl::write_atoms(st, os, opt);
   char buf[88];
-  WRITE("%-80s\n", "END");
+  WRITE("%-80s", "END");
 }
 
 void write_minimal_pdb(const Structure& st, std::ostream& os,
