@@ -9,6 +9,7 @@
 #include "gemmi/small.hpp"    // for SmallStructure
 #include "gemmi/interop.hpp"  // for atom_to_site, mx_to_sx_structure
 #include "gemmi/bessel.hpp"   // for bessel_i1_over_i0
+#include "gemmi/third_party/tao/pegtl/parse_error.hpp" // for parse_error
 
 namespace py = pybind11;
 
@@ -80,6 +81,19 @@ PYBIND11_MODULE(gemmi, mg) {
   mg.doc() = "Python bindings to GEMMI - a library used in macromolecular\n"
              "crystallography and related fields";
   mg.attr("__version__") = GEMMI_VERSION;
+
+  py::register_exception_translator([](std::exception_ptr p) {
+    try {
+      if (p)
+        std::rethrow_exception(p);
+    } catch (const std::system_error &e) {
+      const int errornum = e.code().value();
+      PyErr_SetObject(PyExc_IOError, py::make_tuple(errornum, e.what()).ptr());
+    } catch (const tao::pegtl::parse_error &e) {
+      PyErr_SetString(PyExc_ValueError, e.what());
+    }
+  });
+
   py::module cif = mg.def_submodule("cif", "CIF file format");
   add_cif(cif);
   add_symmetry(mg);
@@ -93,8 +107,8 @@ PYBIND11_MODULE(gemmi, mg) {
   add_ccp4(mg);
   add_sf(mg);
   add_cif_read(cif);
-  add_hkl(mg);
   add_mtz(mg);
+  add_hkl(mg);
   add_monlib(mg);
   add_topo(mg);
   add_alignment(mg);
