@@ -36,16 +36,30 @@ void add_dencalc(py::module& m, const char* name) {
     .def_readwrite("d_min", &DenCalc::d_min)
     .def_readwrite("rate", &DenCalc::rate)
     .def_readwrite("blur", &DenCalc::blur)
-    .def_readwrite("r_cut", &DenCalc::r_cut)
+    .def_readwrite("cutoff", &DenCalc::cutoff)
     .def_readwrite("addends", &DenCalc::addends)
     .def("set_refmac_compatible_blur", &DenCalc::set_refmac_compatible_blur)
     .def("put_model_density_on_grid", &DenCalc::put_model_density_on_grid)
     .def("initialize_grid", &DenCalc::initialize_grid)
     .def("add_model_density_to_grid", &DenCalc::add_model_density_to_grid)
+    .def("add_atom_density_to_grid", &DenCalc::add_atom_density_to_grid)
     .def("add_c_contribution_to_grid", &DenCalc::add_c_contribution_to_grid)
     .def("set_grid_cell_and_spacegroup", &DenCalc::set_grid_cell_and_spacegroup)
     .def("reciprocal_space_multiplier", &DenCalc::reciprocal_space_multiplier)
     .def("mott_bethe_factor", &DenCalc::mott_bethe_factor)
+    .def("estimate_radius", [](const DenCalc &self, const gemmi::Atom &atom){
+        double b;
+        if (!atom.aniso.nonzero())
+            b = atom.b_iso + self.blur;
+        else {
+            gemmi::SMat33<double> aniso_b = atom.aniso.scaled(gemmi::u_to_b()).added_kI(self.blur);
+            // rough estimate, so we don't calculate eigenvalues
+            b = std::max(std::max(aniso_b.u11, aniso_b.u22), aniso_b.u33);
+        }
+        auto coef = Table::get(atom.element);
+        auto precal = coef.precalculate_density_iso(b, self.addends.get(atom.element));
+        return self.estimate_radius(precal, b);
+    })
     ;
 }
 
